@@ -214,7 +214,7 @@ static xdebug_dbgp_cmd dbgp_commands[] = {
 	/* Non standard functions */
 	DBGP_FUNC_ENTRY(xcmd_profiler_name_get,    XDEBUG_DBGP_POST_MORTEM)
 	DBGP_FUNC_ENTRY(xcmd_get_executable_lines, XDEBUG_DBGP_NONE)
-	{ NULL, NULL }
+	{ NULL, NULL, 0, 0 }
 };
 
 /*****************************************************************************
@@ -452,7 +452,7 @@ continue_from_static_root:
 
 			/* First we try a public property */
 			element = prepare_search_key(name, &element_length, "", 0);
-			if (ht && zend_hash_find(ht, element, element_length + 1, (void **) &retval_pp) == SUCCESS) {
+			if (ht && zend_symtable_find(ht, element, element_length + 1, (void **) &retval_pp) == SUCCESS) {
 				retval_p = *retval_pp;
 				goto cleanup;
 			}
@@ -542,6 +542,7 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 								efree(current_classname);
 							}
 							current_classname = NULL;
+							cc_length = 0;
 							current_ce = NULL;
 							if (retval) {
 								st = fetch_ht_from_zval(retval TSRMLS_CC);
@@ -557,6 +558,7 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 								efree(current_classname);
 							}
 							current_classname = NULL;
+							cc_length = 0;
 							current_ce = NULL;
 							if (retval) {
 								current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
@@ -574,6 +576,7 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 								efree(current_classname);
 							}
 							current_classname = NULL;
+							cc_length = 0;
 							if (retval) {
 								current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
 								st = NULL;
@@ -596,7 +599,7 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 						state = 1;
 					}
 					break;
-				case 3:
+				case 3: /* Parsing in [...] */
 					/* Associative arrays */
 					if (*p[0] == '\'' || *p[0] == '"') {
 						state = 4;
@@ -606,10 +609,23 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 					}
 					/* Numerical index */
 					if (*p[0] >= '0' && *p[0] <= '9') {
+						cc_length = 0;
 						state = 6;
 						keyword = *p;
 						type = XF_ST_ARRAY_INDEX_NUM;
 					}
+					/* Numerical index starting with a - */
+					if (*p[0] == '-') {
+						state = 9;
+						keyword = *p;
+					}
+					break;
+				case 9:
+					/* Numerical index starting with a - */
+					if (*p[0] >= '0' && *p[0] <= '9') {
+						state = 6;
+						type = XF_ST_ARRAY_INDEX_NUM;
+					}
 					break;
 				case 4:
 					if (*p[0] == quotechar) {
@@ -621,6 +637,7 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 							efree(current_classname);
 						}
 						current_classname = NULL;
+						cc_length = 0;
 						if (retval) {
 							current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
 							st = fetch_ht_from_zval(retval TSRMLS_CC);
@@ -642,6 +659,7 @@ static zval* get_symbol_contents_zval(char* name, int name_length TSRMLS_DC)
 							efree(current_classname);
 						}
 						current_classname = NULL;
+						cc_length = 0;
 						if (retval) {
 							current_classname = fetch_classname_from_zval(retval, &cc_length, &current_ce TSRMLS_CC);
 							st = fetch_ht_from_zval(retval TSRMLS_CC);
@@ -1949,10 +1967,15 @@ DBGP_FUNC(property_value)
 	if (CMD_OPTION('m')) {
 		options->max_data = strtol(CMD_OPTION('m'), NULL, 10);
 	}
+	if (options->max_data < 0) {
+		options->max_data = old_max_data;
+		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_INVALID_ARGS);
+	}
 	if (add_variable_contents_node(*retval, CMD_OPTION('n'), strlen(CMD_OPTION('n')) + 1, 1, 0, 0, options TSRMLS_CC) == FAILURE) {
 		options->max_data = old_max_data;
 		RETURN_RESULT(XG(status), XG(reason), XDEBUG_ERROR_PROPERTY_NON_EXISTANT);
 	}
+	options->max_data = old_max_data;
 }
 
 static void attach_used_var_with_contents(void *xml, xdebug_hash_element* he, void *options)
@@ -2000,7 +2023,7 @@ static int xdebug_add_filtered_symboltable_var(zval *symbol XDEBUG_ZEND_HASH_APP
 	}
 	if (strcmp("GLOBALS", hash_key->arKey) == 0) { return 0; }
 
-	xdebug_hash_add(tmp_hash, hash_key->arKey, strlen(hash_key->arKey), hash_key->arKey);	
+	xdebug_hash_add(tmp_hash, (char*) hash_key->arKey, strlen(hash_key->arKey), hash_key->arKey);
 
 	return 0;
 }
@@ -2178,7 +2201,8 @@ DBGP_FUNC(xcmd_profiler_name_get)
 DBGP_FUNC(xcmd_get_executable_lines)
 {
 	function_stack_entry *fse;
-	int                   i, depth;
+	unsigned int          i;
+	int                   depth;
 	xdebug_xml_node      *lines, *line;
 
 	if (!CMD_OPTION('d')) {
